var ExifViewerOverlay = {};

(function () {
	'use strict';

	ExifViewerOverlay.ExifViewer = {
		onLoad: function () {
			// initialization code
			this.initialized = true;
		},

		onMenuItemCommand: function () {
			window.open('chrome://exif/content/exif.xul', 'exif_viewer', 'chrome,centerscreen,resizable,menubar');	// ,dependent
		}
	};

	// stuff needed to handle popup menu items properly

	ExifViewerOverlay.initExifOverlay = function () {
		var contextMenu;
		// Firefox:
		contextMenu = document.getElementById('contentAreaContextMenu');
		if (contextMenu) {
			contextMenu.addEventListener('popupshowing', ExifViewerOverlay.showExifMenuItems, false);
		}
		// Thunderbird:
		contextMenu = document.getElementById('messagePaneContext');
		if (contextMenu) {
			contextMenu.addEventListener('popupshowing', ExifViewerOverlay.showExifMenuItems, false);
		}
	}	// initExifOverlay()

	window.addEventListener('load', function (e) { ExifViewerOverlay.ExifViewer.onLoad(e); }, false);
	window.addEventListener('load', ExifViewerOverlay.initExifOverlay, false);

	//show or hide items depending on the context
	ExifViewerOverlay.showExifMenuItems = function () {
		var re_jpg, re_notjpg, 
			bShowViewImage, bShowViewLink, bShowRotateImage, bShowRotateLinkImage;

		re_jpg = new RegExp('\.jp(eg|e|g)(\\?.*)?$', 'i');
		re_notjpg = new RegExp('\.('
									+ 'aiff|amc|asx|au|bmp|fdf|gif|htm|html|'
									+ 'm4a|mid|mov|mp3|mp4|mpeg|mpg|'
									+ 'pdf|pict|png|pntg|qtif|'
									+ 'rm|sdp|spl|swf|txt|'
									+ 'wav|wm|wma|wmv|'
									+ 'xfd|xfdf|xdp|'
									+ '3g2|3gp'
									+ ')(\\?.*)?$', 'i');

		bShowViewImage = (gContextMenu.onImage  &&  !re_notjpg.test(gContextMenu.target.src));
		bShowViewLink = (gContextMenu.onLink  &&  !re_notjpg.test(gContextMenu.link.href));
		bShowRotateImage = (gContextMenu.onImage  &&  re_jpg.test(gContextMenu.target.src));
		bShowRotateLinkImage = (gContextMenu.onLink  &&  re_jpg.test(gContextMenu.link.href));

		// Firefox:
		gContextMenu.showItem('context-item-image-exif', bShowViewImage);
		gContextMenu.showItem('context-item-link-exif', bShowViewLink);
		gContextMenu.showItem('context-item-rotate-image-exif', bShowRotateImage);
		gContextMenu.showItem('context-item-rotate-link-image-exif', bShowRotateLinkImage);

		// Thunderbird:
		gContextMenu.showItem('context-item-image-exif-tb', bShowViewImage);
		gContextMenu.showItem('context-item-link-exif-tb', bShowViewLink);

	/*
		var jpeg_only = false;	// if set to true, suppresses menu item for dynamic urls
		var bShowViewImage = (gContextMenu.onImage  &&  (!jpeg_only || re_jpg.test(gContextMenu.target.src)));
		var bShowViewLink = (gContextMenu.onLink  &&  (!jpeg_only || re_jpg.test(gContextMenu.link.href)));
	*/
	/*
		var menuitem_image = document.getElementById('context-item-image-exif');
		var	menuitem_link = document.getElementById('context-item-link-exif');

		menuitem_image.hidden = false;
		menuitem_link.hidden = false;

		if (gContextMenu.onImage) {
			menuitem_image.hidden = false;
			menuitem_link.hidden = true;
		}
		if (gContextMenu.onLink) {
			menuitem_image.hidden = true;
			menuitem_link.hidden = false;
		}
		if (!(gContextMenu.onImage  ||  gContextMenu.onLink)) {
			menuitem_image.hidden = true;
			menuitem_link.hidden = true;
		}
	*/
	};	// showExifMenuItems()

	ExifViewerOverlay.window = undefined;

	ExifViewerOverlay.handleExifImagePicked = function (url) {
		if (!ExifViewerOverlay.window  ||
				!ExifViewerOverlay.window.ExifViewerApp  ||
				!ExifViewerOverlay.window.ExifViewerApp.exports._handleExifImagePicked) {
			ExifViewerOverlay.window = window.open("chrome://exif/content/exif.xul", "exif_viewer",
										 "chrome,centerscreen,resizable,menubar");
		}	
		ExifViewerOverlay.waitForExifWindow(url);
	};	// handleExifImagePicked()

	ExifViewerOverlay.waitForExifWindow = function (url) {
		if (ExifViewerOverlay.window  &&
				ExifViewerOverlay.window.ExifViewerApp  &&
				ExifViewerOverlay.window.ExifViewerApp.exports._handleExifImagePicked) {
			ExifViewerOverlay.window.ExifViewerApp.exports._handleExifImagePicked(url, ExifViewerOverlay);
			ExifViewerOverlay.window.focus();
	//		ExifViewerOverlay.window = null;	// actually set to null in exif.xul (window.onunload)
		} else {
			window.setTimeout(function () {
				ExifViewerOverlay.waitForExifWindow(url);
			}, 500);
		}
	};	// waitForExifWindow()

	ExifViewerOverlay.handleExifAttachmentPicked = function (url) {
		var i, attachment, attachmentList, selectedAttachments,
			uri, obj, file, messenger, destFilePath;

		try {
			if (url) {
				for (i = 0 ; i < currentAttachments.length ; i += 1) {
					if (url.indexOf('&filename=' + window.escape(currentAttachments[i].displayName)) !== -1) {
						attachment = currentAttachments[i];
						break;
					}
				}
			} else {
				attachmentList = document.getElementById('attachmentList');
				selectedAttachments = attachmentList.selectedItems;
		
				if (selectedAttachments.length < 1) {
					window.alert('Please select an attachment.');
					return;
				} else if (selectedAttachments.length > 1) {
					window.alert('Please select only one attachment.');
					return;
				} else {
					attachment = selectedAttachments[0].attachment;
				}
			}
			
			if (!attachment) {
				window.alert('Nothing to do.');
				return;
			}

	//		window.alert(attachment.contentType + '\n2 ' + attachment.url + '\n3 ' + encodeURIComponent(attachment.displayName)); // + '\n4 ' + attachment.uri);
			uri = attachment.url + '&contentType=' + attachment.contentType + '&filename=' + attachment.displayName;
			obj = Components.classes['@mozilla.org/file/directory_service;1']
						.getService(Components.interfaces.nsIDirectoryService);
			file = Components.classes['@mozilla.org/file/directory_service;1']
						 .getService(Components.interfaces.nsIProperties)
						 .get('TmpD', Components.interfaces.nsIFile);
			messenger = Components.classes['@mozilla.org/messenger;1']
							.getService(Components.interfaces.nsIMessenger);
			destFilePath = messenger.saveAttachmentToFolder(attachment.contentType, attachment.url, 
										'' + encodeURIComponent(attachment.displayName), uri, file);
			window.setTimeout(function () {
				ExifViewerOverlay.handleExifImagePicked('file:///' + destFilePath.path.replace(/\\/g, '/'));
			}, 2000);
			window.setTimeout(function () {
				ExifViewerOverlay.deleteExifTempFile(destFilePath.path.replace(/\\/g, '\\\\'));
			}, 10000);
		} catch (e) {
			window.alert('An error was encountered while trying to handle the attachment. Sorry.');
			window.console.log('Overlay attachment error: ', e);
		}
	}	// handleExifAttachmentPicked()

	ExifViewerOverlay.deleteExifTempFile = function (filename) {
		try {
			var file = Components.classes["@mozilla.org/file/local;1"]
						.createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(filename);
			if (file.exists()) {
				file.remove(false);
			}
		} catch (e) {
			window.console.log('Overlay delete error: ', e);
		}
	}	// deleteExifTempFile()

	ExifViewerOverlay.handleExifRotateImage = function (url) {
		window.open('chrome://exif/content/rotate.html?file=' + window.escape(url) + '&angle=90', 'img',
						'resizable=yes,toolbar=no,status=no,menubar=no,scrollbars=yes');
	}	// handleRotateImage()

	/*
	mailbox:///C|/WINDOWS/Application%20Data/Thunderbird/araskin/jcgidbof.slt/Mail/inbox.allstream.net/Inbox?number=17648837&part=1.2&type=image/jpeg&filename=CenturyOfGrowth.jpg
	@mozilla.org/messenger/mailboxurl;1
	nsIURL.fileBaseName
	nsIURL.fileName
	*/
})();
